<?php

// Sandbox Environment
define('ICL_API_ENDPOINT', 'http://sandbox.icanlocalize.com');

// Production Environment
// define(ICL_API_ENDPOINT, 'https://www.icanlocalize.com');

 
class ICanLocalizeAPI {

    public $siteId; 
    public $accessKey;
    
    /**
    * @param int $siteId Unique identifier for the project
    * @param string $accessKey Access key to authorize project access
    */
    function __construct($siteId=null, $accessKey=null) {
        $this->siteId = $siteId;
        $this->accessKey = $accessKey;    
    }
    
    /**
    * Create a new project at ICanLocalize
    * Can also create a new account, if specified
    *
    * @param array $args Arguments for website creation
    *  Arguments description:
    *		email  User email (all communication will be done with that email address)
    * 		fname  First name
    * 		lname  Last name
    * 		password  Password (optional)
    * 		create_account  
    *            0 – use and existing account (password required), 
    *            1 – create a new account (password will be emailed)
    * 		is_verified  Set to 1
    * 		platform_kind  Set to 2
    * 		interview_translators  Set to 1
    * 		cms_kind  Each type of CMS gets a unique identifier. Contact ICanLocalize.
    * 		cms_description  Name of the content management system
    * 		url  URL of website (include http://)
    * 		title  Title of website (optional)
    * 		description Description of website (optional)
    * 		pickup_type  
    *           0 – ICanLocalize will send XML-RPC notifications 
    *           1 – host system will poll translations
    * 		notifications  Integer value, determining which notifications to send
    *           0 - No notifications
    *           1 – notify when translations are complete
    * 		project_kind  0 – test, 2 – production
    * 		from_languageN  Name of destination language N (projects can have any number of language pairs)
    * 		to_languageN  Name of target language N 
    *
    * @return array(int, str) Site id and access key for the created project
    */
    public function createProject($args) {
        $url = '/websites/create_by_cms.xml';
        $response_xml = $this->sendRequest($url, $args, 'POST');
        
        $this->siteId = (string) $response_xml->website['id'];
        $this->accessKey = (string) $response_xml->website['accesskey'];
        return array($this->siteId, $this->accessKey);
    }

    /**
    * Update project details
    *
    * @param array $args Arguments to be updated.
    * 		Arguments description (only what must be updated should be included):
    * 		url  URL of website (include http://)
    * 		title  Title of website
    * 		description Description of website
    * 		pickup_type  0 - XML-RPC / 1 - Manual Poll
    * 		notifications  0 - No notifications / 1 - notify on translation complete
    * 		project_kind  0 – test, 2 – production
    * 		from_languageN  Name of destination language N 
    * 		to_languageN  Name of target language N
    */
    public function updateProject($args) {
        $url = '/websites/update_by_cms.xml';
        $args['id'] = $this->siteId;
        $args['accesskey'] = $this->accessKey;

        $response_xml = $this->sendRequest($url, $args, 'POST');
    }
    
    /**
    * Send content to translation
    *
    * @param array $args Arguments describing the document
    * 		being sent to translation
    *
    * 	Arguments description:
    * 		orig_language  Name of original language
    * 		to_language1  Name of the destination language
    * 		key  Request's unique identifier (to protect against duplicate requests)
    * 		cms_id  Document's unique identifier (links together document updates)
    * 		title  Title of document
    * 		note  Note to translators (optional)
    * 		doc_count 1
    * 		permlink  Document's URI (optional)
    *       file1[description] cms_request_details
    * 		file1[uploaded_data] The gzip compressed file for translation
    *
    * @return int The id for the translation request created
    */
    public function sendTranslationRequest($args) {
        $url = "/websites/$this->siteId/cms_requests.xml";       
        $file = array('file1[uploaded_data]', $args['file1[uploaded_data]']); 
        $args['accesskey'] = $this->accessKey;
                
        $response_xml = $this->sendRequest($url, $args, 'POST', $file);
        $requestId = (string) $response_xml->result['id'];
        return $requestId;
    }
    
    
    /**
    * Get translation requests list
    * 
    * @param bool $pickup true if only retrieving requests to be fetched
    *
    * @return SimpleXMLElement The list of requests as a XML document
    */
    public function getTranslationRequests($pickup=true) {
        $url = "/websites/$this->siteId/cms_requests.xml?"
            . "accesskey=$this->accessKey";
        $url = $pickup ? $url . '&filter=pickup' : $url;
        $response_xml = $this->sendRequest($url);   
        return $response_xml; 
    }


    /**
    * Download the translation for a given request
    *  
    * @param int $requestId Request identifier
    * @param string $language The name of the translation's target language
    *
    * @return SimpleXMLElement The translated items embedded on the
    *   request content
    */
    public function downloadTranslation($requestId, $language) {
        $url = "/websites/$this->siteId/" 
            . "cms_requests/$requestId/cms_download?"
            . "accesskey=$this->accessKey&language=$language";
            
        $response_xml = $this->sendRequest($url, $args=null, $method='GET',
            $file=null, $gzdecode_response=true);    
        return $response_xml;            
    }  
    
    /**
    * Set the translation status as complete
    * This should be used after fetching a translation, to indicate the 
    *   translation content has being retrieved correctly.
    *
    * @param int $requestId Request identifier
    * @param string $language The name of the translation's target language
    */
    public function setTranslationComplete($requestId, $language) {    
        $args = array('accesskey' => $this->accessKey, 'language' => $language, 
            'status' => 3);
        $url = "/websites/$this->siteId/" 
            . "cms_requests/$requestId/update_status.xml";
        $response_xml = $this->sendRequest($url, $args, 'POST');
    }
    
    /**
    * Set the translated document URL
    * This can be used to add a link to the translation document on the
    *   ICanLocalize translation request
    *
    * @param int $requestId Request identifier
    * @param string $language The name of the translation's target language
    * @param string $permlink Link to the translated document
    */
    public function setTranslationURL($requestId, $language, $permlink) {
         $args = array('accesskey' => $this->accessKey, 'language' => $language, 
            'permlink' => $permlink);
        $url = "/websites/$this->siteId/" 
            . "cms_requests/$requestId/update_permlink.xml";
        $response_xml = $this->sendRequest($url, $args, 'POST');        
    }
    
    
    /**
    * Send the request to ICanLocalize server and retrieve the response
    * 
    * @param string $url Relative url identifying the action
    * @param array $args Arguments to be added to the request
    * @param string $method The method to be used on the request (GET/POST)
    * @param array $files The files to be added to the request
    * @param string gzdecode_response true if the response should is gzipped
    *
    * @throws Exception When error response is returned from the server
    *
    * @return SimpleXMLElement The response's content as a XML document
    */
    protected function sendRequest($url, $args=null, $method='GET', $file=null, 
            $gzdecode_response=false) {            
        require_once 'HTTP/Request2.php';
        
        // Add relative url to ICL url
        $url = ICL_API_ENDPOINT . $url;

        // Set request url, parameters and files
        if ($method == 'POST') {
            $request = new HTTP_Request2($url, HTTP_Request2::METHOD_POST);
        } else {
            $request = new HTTP_Request2($url, HTTP_Request2::METHOD_GET);
        }
        
        $request->addPostParameter($args);        
        if ($file and count($file) > 1) {
            $request->addUpload($file[0], $file[1]);
        }
        
        // Do the call and get response
        $response = $request->send()->getBody();
        
        // Convert response into xml data structure
        if ($gzdecode_response) {
            $response_xml = new SimpleXMLElement(gzdecode($response));
        } else {
            $response_xml = new SimpleXMLElement($response);
            
            // Check response for errors
            if ($response_xml->status['err_code'] != 0) {  
                $error_code = $response_xml->status['err_code'];
                throw new Exception($response_xml->status 
                    . " (error code: $error_code)" );
            }
        }           
        return $response_xml;    
    }
}

//  USAGE EXAMPLES =============================================================

/** ICanLocalize API Usage examples */
class ICanLocalizeUsageExamples {

    function __construct($siteId=null, $accessKey=null) {
        $this->siteId = $siteId;
        $this->accessKey = $accessKey;
        
        $this->requestId = null;
    }
    
    /** Creating a new project */
    function createNewProject($email='iclapiclient@mailinator.com') {
        $icl = new ICanLocalizeAPI();
        $args = array(
            // Account information
            'fname'=>'Iclapi', 
            'lname' => 'Client', 
            'email'=> $email, // Must be unique 
            'create_account' => 1, 
            'is_verified' => 1,  
            // Project information
            'url'=>'http://www.iclapiclient.com', 
            'title' => 'ICL API Client Website', 
            'from_language1'=>'English', 
            'to_language1'=>'Spanish', 
            'pickup_type'=> 1, 
            'notification' => 1,
            // Content Management System information
            'platform_kind'=>2, 
            'cms_kind'=>100, 
            'cms_description'=> 'ICanLocalize API Client'
        );
        $response = $icl->createProject($args);
        $this->siteId = $response[0];
        $this->accessKey = $response[1];
        print "Site id: $this->siteId\n";
        print "Access key: $this->accessKey";
    }

    /** Adding new languages to project */
    function addNewLanguagePair() {
        $icl = new ICanLocalizeAPI($this->siteId, $this->accessKey);
        $args = array(
            'from_language1'=>'English', 
            'to_language1'=>'Spanish', 
            'from_language2'=> 'English',
            'to_language2' => 'Portuguese'
        );
        $icl->updateProject($args);
        print 'Language Pair Added';
    }

    /** Sending content to translation */
    function sendContentToTranslation() {
        $icl = new ICanLocalizeAPI($this->siteId, $this->accessKey);
        $args = array(
            'orig_language'=> 'English', 
            'title'=> 'ICL Partner Translations', 
            'doc_count'=> 1, 
            'key'=> 'abcdefg',  // Request identifier
            'cms_id'=> 'page_1', // Document identifier
            'to_language1'=> 'Spanish', 
            'file1[description]' => 'cms_request_details', 
            // Filepath to the cms request XML gzipped file
            'file1[uploaded_data]' => 'cms_request_details.xml.gz' 
        );
        $this->requestId = $icl->sendTranslationRequest($args);
        print "Request id: $this->requestId";
    }
    
    /** Getting translation requests */
    function getTranslationsList() {
        $icl = new ICanLocalizeAPI($this->siteId, $this->accessKey);        
        
        // Getting all the requests, set pickup to true to get only
        // pending requests
        $response = $icl->getTranslationRequests($pickup=false);
        
        // Printing the first request
        print 'First request id: '. $response->pending_cms_requests->cms_request['id'];
    }
    
    /** Downloading translations 
    NOTE: THIS WILL ONLY WORK IF THE TRANSLATION HAS BEEN COMPLETED    
    */
    function downloadTranslation() {
        $icl = new ICanLocalizeAPI($this->siteId, $this->accessKey);        
        $response = $icl->downloadTranslation($this->requestId, 'Spanish');
        print 'Translation XML';
        print_r($response);
    }
    
    
    /** Setting translation status to complete, after fetching */
    function setTranslationComplete() {
        $icl = new ICanLocalizeAPI($this->siteId, $this->accessKey);        
        $response = $icl->setTranslationComplete($this->requestId, 'Spanish');
        print 'Translation status set to complete';
    } 
}





